from infi import unittest

class ScsiIdTestCase(unittest.TestCase):
    def test_sg0(self):
        from .. import sg_scsi_id as ioctl
        result = ioctl("/dev/sg0")

    def test_sda(self):
        from .. import scsi_ioctl_get_idlun as ioctl
        result = ioctl("/dev/sda")




from infi.instruct import Struct, FixedSizeArray
from infi.instruct import SNInt32 as Int
from infi.instruct import SNInt16 as Short
from infi.instruct import SNInt8 as Byte


# http://tldp.org/HOWTO/SCSI-Generic-HOWTO/g_scsi_id.html
class SG_GET_SCSI_ID(Struct):
    _fields_ = [
                Int("host_no"),
                Int("channel"),
                Int("scsi_id"),
                Int("lun"),
                Int("scsi_type"),
                Short("h_cmd_per_lun"),
                Short("d_queue_depth"),
                FixedSizeArray("unused", 2, Int),
               ]

# http://tldp.org/HOWTO/SCSI-Generic-HOWTO/scsi_g_idlun.html
class SCSI_IDLUN(Struct):
    _fields_ = [
                Int("four_in_one"),
                Int("host_unique_id"),
               ]




from infi import unittest

class SgMapTestCase(unittest.TestCase):

    def test_get_sg_to_hctl_mappings(self):
        from .. import get_sg_to_hctl_mappings as getter
        result = getter()
        self.assertIn("/dev/sg0", result.keys())

    def test_get_sd_to_hctl_mappings(self):
        from .. import get_sd_to_hctl_mappings as getter
        result = getter()
        self.assertIn("/dev/sda", result.keys())

    def test_get_hctl_to_sd_mappings(self):
        from .. import get_hctl_to_sd_mappings as getter
        result = getter()

    def test_sg_to_sd(self):
        from .. import get_sd_from_sg
        self.assertIn(get_sd_from_sg("/dev/sg1"), ["/dev/sda", "/dev/sdb"])




import opcodes
import structures

def ioctl(device_path, op_number, buffer=None):
    with open(device_path) as fd:
        from fcntl import ioctl as _ioctl
        args = [fd, op_number,]
        if buffer is not None:
            args.extend([buffer, True])
        return _ioctl(*args)

def sg_scsi_id(device_path):
    """:returns: a :class:`.SG_GET_SCSI_ID` object"""
    from array import array
    struct_cls = structures.SG_GET_SCSI_ID
    size = struct_cls.min_max_sizeof().max
    buffer = array("B", [0]*size)
    result = ioctl(device_path, opcodes.SG_GET_SCSI_ID, buffer)
    struct = struct_cls.create_from_string(buffer)
    return struct

def scsi_ioctl_get_idlun(device_path):
    """:returns: a :class:`.SCSI_IDLUN` object"""
    from array import array
    struct_cls = structures.SCSI_IDLUN
    size = struct_cls.min_max_sizeof().max
    buffer = array("B", [0]*size)
    result = ioctl(device_path, opcodes.SCSI_IOCTL_GET_IDLUN, buffer)
    struct = struct_cls.create_from_string(buffer)
    return struct




from infi.dtypes.hctl import HCTL

def get_hctl_for_sg_device(device_path):
    from ..ioctl import sg_scsi_id as _ioctl
    struct = _ioctl(device_path)
    return HCTL(struct.host_no, struct.channel, struct.scsi_id, struct.lun)

def get_hctl_for_sd_device(device_path):
    from ..ioctl import scsi_ioctl_get_idlun as _ioctl
    struct = _ioctl(device_path)
    # http://tldp.org/HOWTO/SCSI-Generic-HOWTO/scsi_g_idlun.html
    # "four_in_one" is made up as follows:
    # (scsi_device_id | (lun << 8) | (channel << 16) | (host_no << 24))
    host = (struct.four_in_one >> 24)
    channel = (struct.four_in_one >> 16) & 0xFF
    target = (struct.four_in_one) & 0xFF
    lun = (struct.four_in_one >> 8) & 0xFF
    result = HCTL(host, channel, target, lun)
    return HCTL(host, channel, target, lun)

def get_sg_to_hctl_mappings():
    from glob import glob
    return {device_path:get_hctl_for_sg_device(device_path) for device_path in glob("/dev/sg*")}

def get_sd_to_hctl_mappings():
    from glob import glob
    from os.path import sep
    sd_devices = filter(lambda path: path.split(sep)[-1].isalpha(), glob("/dev/sd*"))
    return {device_path:get_hctl_for_sd_device(device_path) for device_path in sd_devices}

def get_hctl_to_sd_mappings():
    return {hctl:device_path for device_path,hctl in get_sd_to_hctl_mappings().items()}

def get_sd_from_sg(sg):
    hctl = get_hctl_for_sg_device(sg)
    return get_hctl_to_sd_mappings()[hctl]




__version__ = "0.1.6"
__git_commiter_name__ = "Arnon Yaari"
__git_commiter_email__ = "arnony@infinidat.com"
__git_branch__ = '(detached from v0.1.6)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '0671d68a312dfd0a61ab9028715efba3045bbd16'
__git_head_subject__ = 'update homepage'
__git_head_message__ = ''
__git_dirty_diff__ = ''